@@ -4,15 +4,20 @@
 Settings and global variables used in the reading and plotting procedures
 """
 # todo: reduce dependency on globals (e.g flexible if new datasets/versions are added)
-import cartopy.crs as ccrs
+import warnings
 
-# === plot defaults ===
+import cartopy.crs as ccrs
+import matplotlib.pyplot as plt
+
+# PLOT DEFAULT SETTINGS
+# =====================================================
 matplotlib_ppi = 72  # Don't change this, it's a matplotlib convention.
 index_names = ['lat', 'lon', 'gpi']  # Names used for 'latitude' and 'longitude' coordinate.
-time_name = 'time' # not used at the moment, dropped on load
-period_name = 'period' # not used at the moment, dropped on load
+time_name = 'time'  # not used at the moment, dropped on load
+period_name = 'period'  # not used at the moment, dropped on load
 
-dpi = 100  # Resolution in which plots are going to be rendered.
+dpi_min = 100  # Resolution in which plots are going to be rendered.
+dpi_max = 200
 title_pad = 12.0  # Padding below the title in points. default padding is matplotlib.rcParams['axes.titlepad'] = 6.0
 data_crs = ccrs.PlateCarree()  # Default map projection. use one of
 
@@ -23,13 +28,15 @@ naturalearth_resolution = '110m'  # One of '10m', '50m' and '110m'. Finer resolu
 crs = ccrs.PlateCarree()  # projection. Must be a class from cartopy.crs. Note, that plotting labels does not work for most projections.
 markersize = 4  # diameter of Marker in points.
 map_pad = 0.15  # padding relative to map height.
-grid_intervals = [2, 5, 10, 30]  # grid spacing in degree to choose from (plotter will try to make 5 gridlines in the smaller dimension)
-max_title_len = 8 * map_figsize[0]  # maximum length of plot title in chars. if longer, it will be broken in multiple lines.
+grid_intervals = [2, 5, 10,
+                  30]  # grid spacing in degree to choose from (plotter will try to make 5 gridlines in the smaller dimension)
+max_title_len = 8 * map_figsize[
+    0]  # maximum length of plot title in chars. if longer, it will be broken in multiple lines.
 
 # === boxplot_basic defaults ===
 boxplot_printnumbers = True  # Print 'median', 'nObs', 'stdDev' to the boxplot_basic.
 boxplot_height = 6
-boxplot_width = 2.1 # times (n+1), where n is the number of boxes.
+boxplot_width = 2.1  # times (n+1), where n is the number of boxes.
 boxplot_title_len = 8 * boxplot_width  # times the number of boxes. maximum length of plot title in chars.
 tick_size = 8.5
 
@@ -54,22 +61,64 @@ out_metadata_plots = {
 # Colormaps can be set for classes of similar metrics or individually for metrics.
 # Any colormap name can be used, that works with matplotlib.pyplot.cm.get_cmap('colormap')
 # more on colormaps: https://matplotlib.org/users/colormaps.html | https://morphocode.com/the-use-of-color-in-maps/
-# colorcet: http://colorcet.pyviz.org/user_guide/Continuous.html
 
-import colorcet
-import matplotlib.pyplot as plt
 _cclasses = {
     'div_better': plt.cm.get_cmap('RdYlBu'),  # diverging: 1 good, 0 special, -1 bad (pearson's R, spearman's rho')
-    'div_worse': plt.cm.get_cmap('RdYlBu_r'), # diverging: 1 bad, 0 special, -1 good (difference of bias)
+    'div_worse': plt.cm.get_cmap('RdYlBu_r'),  # diverging: 1 bad, 0 special, -1 good (difference of bias)
     'div_neutr': plt.cm.get_cmap('RdYlGn'),  # diverging: zero good, +/- neutral: (bias)
-    'seq_worse': plt.cm.get_cmap('YlGn_r'), #'YlGn_r',  # sequential: increasing value bad (p_R, p_rho, rmsd, ubRMSD, RSS):
-    'seq_better': plt.cm.get_cmap('YlGn'), #'YlGn'  # sequential: increasing value good (n_obs, STDerr)
+    'seq_worse': plt.cm.get_cmap('YlGn_r'),
+    # 'YlGn_r',  # sequential: increasing value bad (p_R, p_rho, rmsd, ubRMSD, RSS):
+    'seq_better': plt.cm.get_cmap('YlGn'),  # 'YlGn'  # sequential: increasing value good (n_obs, STDerr)
 }
 
+_colormaps = {  # from /qa4sm/validator/validation/graphics.py
+    'R': _cclasses['div_better'],
+    'p_R': _cclasses['seq_worse'],
+    'rho': _cclasses['div_better'],
+    'p_rho': _cclasses['seq_worse'],
+    'RMSD': _cclasses['seq_worse'],
+    'BIAS': _cclasses['div_neutr'],
+    'n_obs': _cclasses['seq_better'],
+    'urmsd': _cclasses['seq_worse'],
+    'mse': _cclasses['seq_worse'],
+    'mse_corr': _cclasses['seq_worse'],
+    'mse_bias': _cclasses['seq_worse'],
+    'mse_var': _cclasses['seq_worse'],
+    'RSS': _cclasses['seq_worse'],
+    'tau': _cclasses['div_better'],
+    'p_tau': _cclasses['seq_worse'],
+    'snr': _cclasses['div_better'],
+    'err_std': _cclasses['seq_worse'],
+    'beta': _cclasses['div_neutr'],
+}
+
+# Colorbars for difference plots
+_diff_colormaps = {  # from /qa4sm/validator/validation/graphics.py
+    'R': _cclasses['div_better'],
+    'p_R': _cclasses['div_worse'],
+    'rho': _cclasses['div_better'],
+    'p_rho': _cclasses['div_worse'],
+    'tau': _cclasses['div_better'],
+    'p_tau': _cclasses['div_worse'],
+    'RMSD': _cclasses['div_worse'],
+    'BIAS': _cclasses['div_worse'],
+    'urmsd': _cclasses['div_worse'],
+    'RSS': _cclasses['div_worse'],
+    'mse': _cclasses['div_worse'],
+    'mse_corr': _cclasses['div_worse'],
+    'mse_bias': _cclasses['div_worse'],
+    'mse_var': _cclasses['div_worse'],
+    'snr': _cclasses['div_better'],
+    'err_std': _cclasses['div_worse'],
+    'beta': _cclasses['div_worse'],
+}
+
+# METRICS AND VARIABLES DEFINITIONS
+# =====================================================
 # 0=common metrics, 2=paired metrics (2 datasets), 3=triple metrics (TC, 3 datasets)
 metric_groups = {
     0: ['n_obs'],
-    2: ['R', 'p_R', 'rho','p_rho', 'RMSD', 'BIAS',
+    2: ['R', 'p_R', 'rho', 'p_rho', 'RMSD', 'BIAS',
         'urmsd', 'mse', 'mse_corr', 'mse_bias', 'mse_var',
         'RSS', 'tau', 'p_tau'
         ],
@@ -94,13 +143,13 @@ var_name_ds_sep = {
     3: "{ref_id:d}-{ref_ds}_and_{sat_id0:d}-{sat_ds0}_and_{sat_id1:d}-{sat_ds1}"
 }
 
-# === metadata tempplates ===
-_ref_ds_attr = 'val_ref' # global meta values variable that links to the reference dc
-_ds_short_name_attr = 'val_dc_dataset{:d}' # attribute convention for other datasets
-_ds_pretty_name_attr = 'val_dc_dataset_pretty_name{:d}' # attribute convention for other datasets
-_version_short_name_attr = 'val_dc_version{:d}' # attribute convention for other datasets
-_version_pretty_name_attr = 'val_dc_version_pretty_name{:d}' # attribute convention for other datasets
-_val_dc_variable_pretty_name = 'val_dc_variable_pretty_name{:d}' # attribute convention for variable name
+# === metadata templates ===
+_ref_ds_attr = 'val_ref'  # global meta values variable that links to the reference dc
+_ds_short_name_attr = 'val_dc_dataset{:d}'  # attribute convention for other datasets
+_ds_pretty_name_attr = 'val_dc_dataset_pretty_name{:d}'  # attribute convention for other datasets
+_version_short_name_attr = 'val_dc_version{:d}'  # attribute convention for other datasets
+_version_pretty_name_attr = 'val_dc_version_pretty_name{:d}'  # attribute convention for other datasets
+_val_dc_variable_pretty_name = 'val_dc_variable_pretty_name{:d}'  # attribute convention for variable name
 
 # format should have (metric, ds, ref, other ds)
 _variable_pretty_name = {
@@ -108,26 +157,6 @@ _variable_pretty_name = {
     3: "{} of {} \n against {}, {}"
 }
 
-_colormaps = {  # from /qa4sm/validator/validation/graphics.py
-    'R': _cclasses['div_better'],
-    'p_R': _cclasses['seq_worse'],
-    'rho': _cclasses['div_better'],
-    'p_rho': _cclasses['seq_worse'],
-    'RMSD': _cclasses['seq_worse'],
-    'BIAS': _cclasses['div_neutr'],
-    'n_obs': _cclasses['seq_better'],
-    'urmsd': _cclasses['seq_worse'],
-    'mse': _cclasses['seq_worse'],
-    'mse_corr': _cclasses['seq_worse'],
-    'mse_bias': _cclasses['seq_worse'],
-    'mse_var': _cclasses['seq_worse'],
-    'RSS': _cclasses['seq_worse'],
-    'tau':_cclasses['div_better'],
-    'p_tau': _cclasses['seq_worse'],
-    'snr': _cclasses['div_better'],
-    'err_std': _cclasses['seq_worse'],
-    'beta': _cclasses['div_neutr'],
-}
 # check if every metric has a colormap
 for group in metric_groups.keys():
     assert all([m in _colormaps.keys() for m in metric_groups[group]])
@@ -158,27 +187,6 @@ _metric_mask_range = {
     'err_std': [0, None],  # values below 0 exit but should be marked
 }
 
-# Colorbars for difference plots
-_diff_colormaps = {  # from /qa4sm/validator/validation/graphics.py
-    'R': _cclasses['div_better'],
-    'p_R': _cclasses['div_worse'],
-    'rho': _cclasses['div_better'],
-    'p_rho': _cclasses['div_worse'],
-    'tau': _cclasses['div_better'],
-    'p_tau': _cclasses['div_worse'],
-    'RMSD': _cclasses['div_worse'],
-    'BIAS': _cclasses['div_worse'],
-    'urmsd': _cclasses['div_worse'],
-    'RSS': _cclasses['div_worse'],
-    'mse': _cclasses['div_worse'],
-    'mse_corr': _cclasses['div_worse'],
-    'mse_bias': _cclasses['div_worse'],
-    'mse_var': _cclasses['div_worse'],
-    'snr': _cclasses['div_better'],
-    'err_std': _cclasses['div_worse'],
-    'beta': _cclasses['div_worse'],
-}
-
 # check if every metric has a colormap
 for group in metric_groups.keys():
     assert all([m in _colormaps.keys() for m in metric_groups[group]])
@@ -218,6 +226,8 @@ _metric_units = {  # from /qa4sm/validator/validation/graphics.py
     'ESA_CCI_SM_combined': r'm³/m³',
     'ESA_CCI_SM_passive': r'm³/m³',
     'SMOS': r'm³/m³',
+    'CGLS_CSAR_SSM1km': r'percentage of saturation',
+    'CGLS_SCATSAR_SWI1km': r'percentage of saturation',
 }
 
 # label name for all metrics
@@ -242,24 +252,6 @@ _metric_name = {  # from /qa4sm/validator/validation/globals.py
     'beta': 'TC scaling coefficient',
 }
 
-# === pretty names for datasets ===
-# fallback for dataset pretty names in case they are not in the metadata
-_dataset_pretty_names = {  # from qa4sm\validator\fixtures\datasets.json
-    'ISMN': r'ISMN',
-    'C3S': r'C3S',
-    'GLDAS': r'GLDAS',
-    'ASCAT': r'H-SAF ASCAT SSM CDR',
-    'SMAP': r'SMAP level 3',
-    'ERA5': r'ERA5',
-    'ERA5_LAND': r'ERA5-Land',
-    'ESA_CCI_SM_active': r'ESA CCI SM active',
-    'ESA_CCI_SM_combined': r'ESA CCI SM combined',
-    'ESA_CCI_SM_passive': r'ESA CCI SM passive',
-    'SMOS': r'SMOS IC',
-    'CGLS_CSAR_SSM1km': r'CGLS S1 SSM',
-    'CGLS_SCATSAR_SWI1km': r'CGLS SCATSAR SWI',
-}
-
 # label format for all metrics for HTML rendering
 _metric_description_HTML = {  # from /qa4sm/validator/validation/graphics.py
     'R': ' [-]',
@@ -299,14 +291,32 @@ _metric_units_HTML = {  # from /qa4sm/validator/validation/graphics.py
     'CGLS_SCATSAR_SWI1km': '% sat',
 }
 
-# Backups
-# -------
-# to fallbsck to in case the dataset attributes in the .nc file are missing some entries. Sould have variable short
-# name as keys as these should be always available in the template.
+# BACKUPS
+# =====================================================
+# to fallback to in case the dataset attributes in the .nc file are
+# missing some entries. Sould have variable short name as keys as these
+# should be always available in the template.
+
+# fallback for dataset pretty names in case they are not in the metadata
+_dataset_pretty_names = {  # from qa4sm\validator\fixtures\datasets.json
+    'ISMN': r'ISMN',
+    'C3S': r'C3S',
+    'GLDAS': r'GLDAS',
+    'ASCAT': r'H-SAF ASCAT SSM CDR',
+    'SMAP': r'SMAP level 3',
+    'ERA5': r'ERA5',
+    'ERA5_LAND': r'ERA5-Land',
+    'ESA_CCI_SM_active': r'ESA CCI SM active',
+    'ESA_CCI_SM_combined': r'ESA CCI SM combined',
+    'ESA_CCI_SM_passive': r'ESA CCI SM passive',
+    'SMOS': r'SMOS IC',
+    'CGLS_CSAR_SSM1km': r'CGLS S1 SSM',
+    'CGLS_SCATSAR_SWI1km': r'CGLS SCATSAR SWI',
+}
 
 # available backups
 _backups = {
-    "_version_short_name_attr" : "_dataset_version_pretty_names",
+    "_version_short_name_attr": "_dataset_version_pretty_names",
     "_val_dc_variable_pretty_name": "_dataset_variable_names"
 }
 
@@ -336,7 +346,7 @@ _dataset_version_pretty_names = {  # from qa4sm\validator\fixtures\versions.json
     "SMOS_105_DES": "V.105 Descending",
     "ERA5_test": " ERA5 test",
     "ERA5_20190613": "v20190613",
-    "ERA5_LAND_V20190904" : "v20190904",
+    "ERA5_LAND_V20190904": "v20190904",
     "ERA5_LAND_TEST": "ERA5-Land test",
     "CGLS_CSAR_SSM1km_V1_1": "v1_1",
     "CGLS_SCATSAR_SWI1km_V1_0": "v1_0",
@@ -369,14 +379,77 @@ _dataset_variable_names = {  # from qa4sm\validator\fixtures\versions.json
     "SMOS_105_DES": "soil moisture",
     "ERA5_test": "svwl1",
     "ERA5_20190613": "svwl1",
-    "ERA5_LAND_V20190904" : "svwl1",
+    "ERA5_LAND_V20190904": "svwl1",
     "ERA5_LAND_TEST": "svwl1",
     "CGLS_CSAR_SSM1km_V1_1": "soil moisture",
     "CGLS_SCATSAR_SWI1km_V1_0": "SWI",
 }
 
-# Metadata statics
-# ----------------
+
+# ----------- fallback for resolution information -----------------------
+def get_resolution_info(dataset, raise_error=False):
+    # function to get resolution information with possibility to raise error
+    # This info is first looked for in the validation file; if not present,
+    # this function works as fallback unless the specific dataset is not
+    # listed in the lookup table, in which case an error can be rased, according
+    # to 'raise_error'
+
+    resolution = {  # from /qa4sm/validator/fixtures/datasets.json
+        'ISMN': None,
+        'C3S': 0.25,
+        'GLDAS': 0.25,
+        'ASCAT': 12.5,
+        'SMAP': 36,
+        'ERA5': 0.25,
+        'ERA5_LAND': 0.1,
+        'ESA_CCI_SM_active': 0.25,
+        'ESA_CCI_SM_combined': 0.25,
+        'ESA_CCI_SM_passive': 0.25,
+        'SMOS': 25,
+        'CGLS_CSAR_SSM1km': 1,
+        'CGLS_SCATSAR_SWI1km': 1,
+    }
+
+    # fallback for resolution unit information
+    resolution_units = {  # from /qa4sm/validator/fixtures/datasets.json
+        'ISMN': 'point',
+        'C3S': 'deg',
+        'GLDAS': 'deg',
+        'ASCAT': 'km',
+        'SMAP': 'km',
+        'ERA5': 'deg',
+        'ERA5_LAND': 'deg',
+        'ESA_CCI_SM_active': 'deg',
+        'ESA_CCI_SM_combined': 'deg',
+        'ESA_CCI_SM_passive': 'deg',
+        'SMOS': 'km',
+        'CGLS_CSAR_SSM1km': 'km',
+        'CGLS_SCATSAR_SWI1km': 'km',
+    }
+
+    try:
+        dataset_res = resolution[dataset]
+        dataset_units = resolution_units[dataset]
+
+        return dataset_res, dataset_units
+
+    except KeyError:
+        if raise_error:
+            raise KeyError(
+                f"The dataset {dataset} has not been specified in {__name__}"
+            )
+
+        else:
+            warnings.warn(
+                f"The dataset {dataset} has not been specified in {__name__}. "
+                f"Set 'raise_error' to True to raise an exception for this case."
+            )
+
+            return None, None
+
+
+# METADATA STATICS
+# =====================================================
 # information needed for plotting the metadata-boxplots
 
 lc_classes = {
@@ -456,25 +529,25 @@ climate_classes = {
     "ET": "Polar",
     "EF": "Polar",
     "W": "Water",
-    "Mediterranean":"Mediterranean",
+    "Mediterranean": "Mediterranean",
 }
 
 metadata = {
-    "clay_fraction":("clay fraction", None, "continuous", "[% weight]"),
-    "climate_KG":("Koeppen-Geiger climate class", climate_classes, "classes", None),
-    "climate_insitu":("climate in-situ", climate_classes, "classes", None),
-    "elevation":("elevation", None, "continuous", "[m]"),
-    "instrument":("instrument type", None, "discrete", None),  # todo: improve labels (too packed)
-    "lc_2000":("land cover class (2000)", lc_classes, "classes", None),
-    "lc_2005":("land cover class (2005)", lc_classes, "classes", None),
-    "lc_2010":("land cover class (2010)", lc_classes, "classes", None),
-    "lc_insitu":("land cover class in-situ", lc_classes, "classes", None), # todo: handle custom names
-    "network":("network", None, "discrete", None),
-    "organic_carbon":("concentration of organic carbon", None, "continuous", "[% weight]"),
-    "sand_fraction":("sand fraction", None, "continuous", "[% weight]"),
-    "saturation":("saturation", None, "continuous", "[m³/m³]"),
-    "silt_fraction":("silt fraction", None, "continuous", "[% weight]"),
-    "station":("station", None, "discrete", None),
+    "clay_fraction": ("clay fraction", None, "continuous", "[% weight]"),
+    "climate_KG": ("Koeppen-Geiger climate class", climate_classes, "classes", None),
+    "climate_insitu": ("climate in-situ", climate_classes, "classes", None),
+    "elevation": ("elevation", None, "continuous", "[m]"),
+    "instrument": ("instrument type", None, "discrete", None),  # todo: improve labels (too packed)
+    "lc_2000": ("land cover class (2000)", lc_classes, "classes", None),
+    "lc_2005": ("land cover class (2005)", lc_classes, "classes", None),
+    "lc_2010": ("land cover class (2010)", lc_classes, "classes", None),
+    "lc_insitu": ("land cover class in-situ", lc_classes, "classes", None),  # todo: handle custom names
+    "network": ("network", None, "discrete", None),
+    "organic_carbon": ("concentration of organic carbon", None, "continuous", "[% weight]"),
+    "sand_fraction": ("sand fraction", None, "continuous", "[% weight]"),
+    "saturation": ("saturation", None, "continuous", "[m³/m³]"),
+    "silt_fraction": ("silt fraction", None, "continuous", "[% weight]"),
+    "station": ("station", None, "discrete", None),
     "instrument_depthfrom": ("upper depth", None, "continuous", "[m]"),
     "instrument_depthto": ("lower depth", None, "continuous", "[m]"),
     # --- generated during the image initialization:
