@@ -58,7 +58,7 @@ class QA4SMImg(object):
         index_names : list, optional (default: ['lat', 'lon'] - as in globals.py)
             Names of dimension variables in x and y direction (lat, lon).
         load_data: bool, default is True
-            if true, initialize all the datasets, variables and metadata
+            if true, initialise all the datasets, variables and metadata
         engine: str, optional (default: h5netcdf)
             Engine used by xarray to read data from file.
         """
@@ -112,6 +112,22 @@ class QA4SMImg(object):
         else:
             return ds
 
+    @property
+    def res_info(self) -> dict:
+        # Return the resolution of the validation as a dictionary {units: str, value: float/int}
+        try:
+            res_units = self.ds.attrs["val_resolution_unit"]
+            resolution = self.ds.attrs["val_resolution"]
+
+        # fallback to globals if the output attribute is missing
+        except KeyError:
+            resolution, res_units = globals.get_resolution_info(
+                self.datasets.ref['short_name'],
+                raise_error=False,
+            )
+
+        return {"value": resolution, "units": res_units}
+
     @property
     def has_CIs(self):
         """True if the validation result contains confidence intervals"""
@@ -490,7 +506,7 @@ class QA4SMImg(object):
         stats_df.sort_values(by='Group', inplace=True)
         # format the numbers for display
         stats_df = stats_df.applymap(_format_floats)
-        stats_df.sort_index(inplace=True)
+        stats_df.drop(labels='Group', axis='columns', inplace=True)
 
         return stats_df
 
