@@ -614,6 +614,8 @@ def boxplot(
         Resolution for raster graphic output. The default is globals.dpi.
     spacing : float, optional.
         Space between the central boxplot and the CIs. Default is 0.3
+    axis : matplotlib Axis obj.
+        if provided, the plot will be shown on it
 
     Returns
     -------
@@ -1258,7 +1260,7 @@ def mapplot(
         add_cbar=True,
         label=None,
         figsize=globals.map_figsize,
-        dpi=globals.dpi,
+        dpi=globals.dpi_min,
         diff_map=False,
         **style_kwargs
 ) -> tuple:
@@ -1276,7 +1278,7 @@ def mapplot(
                 short_name of the reference dataset (read from netCDF file)
         ref_grid_stepsize : float or None, optional (None by default)
                 angular grid stepsize, needed only when ref_is_angular == False,
-        plot_extent : tuple
+        plot_extent : tuple or None
                 (x_min, x_max, y_min, y_max) in Data coordinates. The default is None.
         colormap :  Colormap, optional
                 colormap to be used.
@@ -1287,7 +1289,7 @@ def mapplot(
         add_cbar : bool, optional
                 Add a colorbar. The default is True.
         label : str, optional
-            Label of the y axis, describing the metric. If None, a label is autogenerated from metadata.
+            Label of the y-axis, describing the metric. If None, a label is autogenerated from metadata.
             The default is None.
         figsize : tuple, optional
             Figure size in inches. The default is globals.map_figsize.
@@ -1376,16 +1378,16 @@ def plot_spatial_extent(
     ----------
     polys : dict
         dictionary with shape {name: shapely.geometry.Polygon}
-    ref_points : 2D array
-        array of lon, lat for the reference points positions
+    ref_points : 2D array of lon, lat for the reference points positions
     overlapping : bool, dafault is False.
         Whether the polygons have an overlap
     intersection_extent : tuple | None
         if given, corresponds to the extent of the intersection. Shape (minlon, maxlon, minlat, maxlat)
     reg_grid : bool, default is False,
         plotting oprion for regular grids (satellites)
+    grid_stepsize:
     """
-    fig, ax, cax = init_plot(figsize=globals.map_figsize, dpi=globals.dpi)
+    fig, ax, cax = init_plot(figsize=globals.map_figsize, dpi=globals.dpi_min)
     legend_elements = []
     # plot polygons
     for n, items in enumerate(polys.items()):
@@ -1488,3 +1490,45 @@ def plot_spatial_extent(
         framealpha=0.4,
         edgecolor='black'
     )
+
+
+def _res2dpi_fraction(res, units):
+    # converts a certain validation resolution to a 0-1 fraction
+    # indicating the output quality
+    # equivalent min/max ranges for km and degrees based on
+    # available datasets, approximated
+    res_range = {
+        "km": [1, 36],
+        "deg": [0.01, 0.33],
+    }
+
+    fraction = (res - min(res_range[units])) / (max(res_range[units]) - min(res_range[units]))
+
+    return (1 - fraction)**2
+
+
+def _extent2dpi_fraction(extent):
+    # converts a certain validation extent to a 0-1 fraction
+    # indicating the output quality
+    max_extent = 360 * 110
+    actual_extent = (extent[1]-extent[0]) * (extent[3]-extent[2])
+
+    return actual_extent / max_extent
+
+
+def output_dpi(
+        res,
+        units,
+        extent,
+        dpi_min=globals.dpi_min,
+        dpi_max=globals.dpi_max
+) -> float:
+    # get ouput dpi based on image extent and validation resolution
+    # dpi = SQRT(extent_coeff^2 + res_coeff^2)
+    dpi_vec = _extent2dpi_fraction(extent)**2 + _res2dpi_fraction(res, units)**2
+    dpi_vec = np.sqrt(dpi_vec)
+    dpi_fraction = dpi_vec / np.sqrt(2)
+
+    dpi = dpi_min + (dpi_max-dpi_min) * dpi_fraction
+
+    return dpi
